/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics;

import java.io.File;


/**
 * <p>
 * StatsReporting interface defines the contract to aggregate and render the statistical report.
 * Result is fed into this interface one by one and the file report is then generated. There is a
 * way to reset (clear) the current aggregated results.
 * </p>
 *
 * <p>
 * Contract Constructor: All implementation should have a constructor with a Property as an
 * argument in order to reflect from configuration file. This requirement can be dropped if it's
 * not necessary to use this way.
 * </p>
 *
 * <p>
 * Thread Safety: There is no thread-safety requirement/recommendation for the implementations.
 * </p>
 *
 * @author WishingBone
 * @version 1.0
 */
public interface StatsReporting {
    /**
     * <p>
     * Aggregate the result into the reporting implementation. The results are kept by the
     * reporting itself.
     * </p>
     *
     * @param processor the processor that generated the result
     * @param file the file that is processed
     * @param result the processing result
     *
     * @throws NullPointerException if any  argument is null
     * @throws StatsReportingException if file is not in base dir
     */
    public void aggregate(FileProcessor processor, File file, StatsResult result)
        throws StatsReportingException;

    /**
     * <p>
     * Generate the report with all the aggregated results.
     * </p>
     *
     * @return the generated report
     *
     * @throws StatsReportingException if error occurs
     */
    public String generateReport() throws StatsReportingException;

    /**
     * <p>
     * Clear all the aggregated result and set the base dir (which is nullable).
     * </p>
     *
     * @param baseDirectory the base dir to reset
     *
     * @throws IllegalArgumentException if baseDirectory is not a valid dir
     */
    public void reset(File baseDirectory);
}
