/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.util.config.Property;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * FileNameMatcher concrete implementation that matches one or more file extensions. The extensions
 * are matched in a case-insensitive manner.
 * </p>
 *
 * <p>
 * Configuration: This class can be constructed from a Property instance (reflection contract). The
 * supported property is "extensions" which is potentially multi-valued.
 * </p>
 *
 * <p>
 * Thread Safety: This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class ExtensionMatcher extends FileNameMatcher {
    /**
     * <p>
     * Configuration key that contains multiple file extensions to match. Though extensions are
     * saved as they are the matching is case-insensitive. Empty is allowed in order to match a
     * file with no extension.
     * </p>
     */
    public static final String EXTENSIONS_PROPERTY_KEY = "extensions";

    /**
     * <p>
     * The char that separates the extension.
     * </p>
     */
    private static final char EXTENSION_SEPARATOR = '.';

    /**
     * <p>
     * A list of file extensions (String) to match. This list is initialized in constructor and is
     * immutable.
     * </p>
     */
    private final List extensions = new ArrayList();

    /**
     * <p>
     * Constructor with single extension. The extension can be empty.
     * </p>
     *
     * @param extension a single file extension
     *
     * @throws NullPointerException if extension is null
     */
    public ExtensionMatcher(String extension) {
        // check for null
        MatcherHelper.checkNull(extension, "extension");

        // add the extension
        extensions.add(extension.toUpperCase());
    }

    /**
     * <p>
     * Constructor with multiple extensions. The extensions in the list can be empty.
     * </p>
     *
     * @param extensions the extensions to match
     *
     * @throws NullPointerException if extensions is null
     * @throws IllegalArgumentException if extensions is empty or contains null or non-String
     *         instance
     */
    public ExtensionMatcher(List extensions) {
        // check for null
        MatcherHelper.checkNull(extensions, "extensions");

        // check for empty
        if (extensions.size() == 0) {
            throw new IllegalArgumentException("extensions should not be empty list.");
        }

        // check for non-String element.
        for (Iterator itr = extensions.iterator(); itr.hasNext();) {
            Object extenstion = itr.next();

            if (extenstion == null) {
                throw new IllegalArgumentException("extensions contains null element.");
            } else if (!(extenstion instanceof String)) {
                throw new IllegalArgumentException("extensions contains non-String instance.");
            } else {
                // add the String element to inner extensions list
                this.extensions.add(((String) extenstion).toUpperCase());
            }
        }
    }

    /**
     * <p>
     * Constructor that loads extensions from a property. An "extensions" sub-property should exist
     * in the given property.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if property does not contain an extensions sub-property or it
     *         does not hold any values
     */
    public ExtensionMatcher(Property property) throws ConfigurationException {
        String[] values = MatcherHelper.getProperty(property, EXTENSIONS_PROPERTY_KEY);

        // convert all values to uppercase and add to inner list
        for (int i = 0; i < values.length; ++i) {
            extensions.add(values[i].toUpperCase());
        }
    }

    /**
     * <p>
     * Checks whether the filename matches any of the extensions. The extension is extracted after
     * the last dot in the filename. If the filename does not contain a dot, the extension is
     * empty. Matching is case-insensitive.
     * </p>
     *
     * @param filename the filename to match
     *
     * @return whether the filename matches
     */
    protected boolean match(String filename) {
        //extract the extension
        String extension = null;
        int position = filename.lastIndexOf(EXTENSION_SEPARATOR);

        if (position >= 0) {
            extension = filename.substring(position + 1);
        } else {
            extension = "";
        }

        // return whether the filename matchs
        return extensions.contains(extension.toUpperCase());
    }
}
