/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.reporting;

import java.io.File;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.util.config.Property;


/**
 * <p>
 * This class provides a StatsReporting implementation base. It aggregates results from each
 * processor and classifies them based on single file, directory or alias. It also provides an
 * overall stats. It defers the actual rendering logic to concrete implementations
 * (BasicStatsReporting and XmlStatsReporting).
 * </p>
 *
 * <p>
 * Configuration: This class has a constructor with Property for extension purpose. There is no
 * real logic in this release.
 * </p>
 *
 * <p>
 * Thread Safety: This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class AbstractStatsReporting implements StatsReporting {
    /**
     * <p>
     * The stats mapped to each file. The file path is relative to base dir or absolute if no base
     * dir. Its reference never changes.
     * </p>
     */
    private final StatsCollection fileStats = new StatsCollection();

    /**
     * <p>
     * The stats mapped to each directory. The dir path is relative to base dir or absolute if no
     * base dir. Its reference never changes.
     * </p>
     */
    private final StatsCollection directoryStats = new StatsCollection();

    /**
     * <p>
     * The stats mapped to each alias.
     * </p>
     */
    private final StatsCollection aliasStats = new StatsCollection();

    /**
     * <p>
     * The overall stats. It is created anew when reporting resets.
     * </p>
     */
    private StatsResult overallStats = new StatsResult();

    /**
     * <p>
     * The base dir for the reporting. This variable is nullable (meaning all paths are absolute).
     * </p>
     */
    private File baseDirectory = null;

    /**
     * <p>
     * The base path string. Retrieved from baseDirectory.
     * </p>
     */
    private String basePath = null;


    /**
     * <p>
     * Default constructor.
     * </p>
     */
    protected AbstractStatsReporting() {
        // empty
    }

    /**
     * <p>
     * Constructor with property. This does not have any functionality in the current release and
     * is kept for extension purpose.
     * </p>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    protected AbstractStatsReporting(Property property) throws ConfigurationException {
        if (property == null) {
            throw new NullPointerException("property should not be null");
        }
    }

    /**
     * <p>
     * Aggregate a result into the reporting.
     * </p>
     *
     * <p>
     * 1) Get the file name. If base dir exists, strip the base portion off. 2) Aggregate the
     * result against file, each path it is in, alias and the overall stats. 3) Notice the dir
     * stats does not need to include the base dir (since it's same as overall).
     * </p>
     *
     * @param processor the processor that generated the result
     * @param file the file that is processed
     * @param result the result
     *
     * @throws NullPointerException if processor or file or result is null
     * @throws StatsReportingException if file is not in base dir
     */
    public void aggregate(FileProcessor processor, File file, StatsResult result)
        throws StatsReportingException {

        if (processor == null) {
            throw new NullPointerException("processor should not be null");
        }
        if (file == null) {
            throw new NullPointerException("file should not be null");
        }
        if (result == null) {
            throw new NullPointerException("result should not be null");
        }

        // strip base directory from given file
        String fileName = stripBaseDirectory(file);

        // update the file stats
        fileStats.add(fileName, result);

        String directoryName = new File(fileName).getParent();
        if (directoryName != null) {
            // update the directory stats, if the file is not the base directory
            directoryStats.add(directoryName, result);
        }

        // update the alias stats
        aliasStats.add(processor.getAlias(), result);

        // update the overall stats
        overallStats.aggregate(result);
    }


    /**
     * Strips the base directory from the given file.
     *
     * @param file the file to strip
     * @throws StatsReportingException if file is invalid or not in base dir
     * @return the stripped path
     */
    private String stripBaseDirectory(File file) throws StatsReportingException {

        // return the absolute path if baseDirectory is null
        if (baseDirectory == null) {
            return file.getAbsolutePath();
        }

        String path = file.getAbsolutePath();
        if (!path.startsWith(basePath)) {
            // throw an exception if the file is not in base dir
            throw new StatsReportingException("The file is not in base dir.");
        }

        String name = path.substring(basePath.length());
        if (name.length() == 0) {
            // throw an exception if the file is the base dir
            throw new StatsReportingException("the file can not be base directory");
        }

        return name;
    }

    /**
     * <p>
     * Reset the reporting. Base dir can be null when absolute paths are desirable.
     * </p>
     *
     * <p>
     * 1) Clear all the collections. 2) Create a new overall StatsResult. 3) Set the base dir.
     * </p>
     *
     * @param baseDirectory the base dir to use
     *
     * @throws IllegalArgumentException if baseDirectory is not a valid dir
     */
    public void reset(File baseDirectory) {

        // reset base directory
        if (baseDirectory == null) {
            this.basePath = null;
            this.baseDirectory = null;
        } else {
            if (!baseDirectory.isDirectory()) {
                throw new IllegalArgumentException("baseDirectory is not a valid dir");
            }
            this.basePath = baseDirectory.getAbsoluteFile() + File.separator;
            this.baseDirectory = baseDirectory;
        }

        // clear all stats
        fileStats.clear();
        directoryStats.clear();
        aliasStats.clear();

        // reset overall stats
        overallStats = new StatsResult();
    }

    /**
     * <p>
     * Get the file stats.
     * </p>
     *
     * @return the file stats
     */
    public StatsCollection getFileStats() {
        return fileStats;
    }

    /**
     * <p>
     * Get the directory stats.
     * </p>
     *
     * @return the directory stats
     */
    public StatsCollection getDirectoryStats() {
        return directoryStats;
    }

    /**
     * <p>
     * Get the alias stats.
     * </p>
     *
     * @return the alias stats
     */
    public StatsCollection getAliasStats() {
        return aliasStats;
    }

    /**
     * <p>
     * Get a copy of overall stats.
     * </p>
     *
     * @return the overall stats
     */
    public StatsResult getOverallStats() {
        return new StatsResult(overallStats);
    }

    /**
     * <p>
     * Get the base dir.
     * </p>
     *
     * @return the base dir
     */
    public File getBaseDirectory() {
        return baseDirectory;
    }
}
