/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;
import junit.framework.TestCase;
import java.io.File;
import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * Unit test cases for ExtensionMatcher.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ExtensionMatcherTests extends TestCase {
    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.matcher.unittest";

    /**
     * The config file for this test.
     */
    private static final String CONFIG_FILE = "matcher/matcher_config.xml";

    /**
     * An ExtensionMatcher instance to test.
     */
    private ExtensionMatcher matcher = null;

    /**
     * Load the configuration before testing.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        TestHelper.loadConfigFile(CONFIG_FILE, NAMESPACE);
        matcher = new ExtensionMatcher("foo");
    }

    /**
     * Remove the configuration after testing.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {
        TestHelper.removeNamespace(NAMESPACE);
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(String).
     * </p>
     */
    public void testExtensionMatcher1() {
        assertTrue("Unable to instantiate ExtensionMatcher.",
            new ExtensionMatcher("foo") instanceof FileNameMatcher);
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(String) with null argument. NullPointerException
     * should be thrown.
     * </p>
     */
    public void testExtensionMatcher2() {
        try {
            new ExtensionMatcher((String) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtensionMatcher3() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "extension_matcher");
        assertTrue("Unable to instantiate ExtensionMatcher.",
            new ExtensionMatcher(property) instanceof FileNameMatcher);
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(Property) with null argument.
     * NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtensionMatcher4() throws Exception {
        try {
            new ExtensionMatcher((Property) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtensionMatcher5() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "extension_matcher_invalid1");

        try {
            new ExtensionMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtensionMatcher6() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "extension_matcher_invalid2");

        try {
            new ExtensionMatcher(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(List).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtensionMatcher7() throws Exception {
        List extensions = new ArrayList();
        extensions.add("foo");
        extensions.add("");
        assertTrue("Unable to instantiate ExtensionMatcher.",
            new ExtensionMatcher(extensions) instanceof FileNameMatcher);
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(List) with null argument. NullPointerException
     * should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtensionMatcher8() throws Exception {
        try {
            new ExtensionMatcher((List) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(List) with empty list. IllegalArgumentException
     * should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtensionMatcher9() throws Exception {
        try {
            new ExtensionMatcher(new ArrayList());
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - ExtensionMatcher(List) with invalid list.
     * IllegalArgumentException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testExtensionMatcher10() throws Exception {
        List extensions = new ArrayList();
        extensions.add("foo");
        extensions.add(new ArrayList());

        try {
            new ExtensionMatcher(extensions);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the match method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch1() throws Exception {
        try {
            matcher.match((File) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the match method of a general extension mather.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch2() throws Exception {
        // create a matcher that accepts "foo".
        matcher = new ExtensionMatcher("foO");

        assertTrue("The result should be true", matcher.match(new File("test.foo")));
        assertTrue("The result should be true", matcher.match(new File("test.bar.FOo")));
        assertFalse("The result should be false", matcher.match(new File("tests.bar")));
        assertFalse("The result should be false", matcher.match(new File("tests.foo.bar")));
        assertTrue("The result should be true", matcher.match(new File(".Foo")));
        assertFalse("The result should be false", matcher.match(new File("foo")));
        assertFalse("The result should be false", matcher.match(new File("foo.")));
    }

    /**
     * <p>
     * Tests the match method of an empty extension mather.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch3() throws Exception {
        // create a matcher that accepts empty extension.
        matcher = new ExtensionMatcher("");

        assertTrue("The result should be true", matcher.match(new File("test.")));
        assertTrue("The result should be true", matcher.match(new File("test")));
        assertFalse("The result should be false", matcher.match(new File(".test")));
        assertTrue("The result should be true", matcher.match(new File("")));
        assertTrue("The result should be true", matcher.match(new File(".")));
        assertTrue("The result should be true", matcher.match(new File("..")));
    }

    /**
     * <p>
     * Tests the match method of multi-extension mather.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch4() throws Exception {
        // create a matcher that accepts "foo" and "".
        List extensions = new ArrayList();
        extensions.add("foo");
        extensions.add("");
        matcher = new ExtensionMatcher(extensions);

        assertTrue("The result should be true", matcher.match(new File("foo")));
        assertTrue("The result should be true", matcher.match(new File("bar")));
        assertFalse("The result should be false", matcher.match(new File("foo.bar")));
        assertTrue("The result should be true", matcher.match(new File("bar.foo")));
    }

    /**
     * <p>
     * Tests the match method of multi-extension mather.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testMatch5() throws Exception {
        // create a matcher that accepts "foo" and "".
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "extension_matcher");
        matcher = new ExtensionMatcher(property);

        assertTrue("The result should be true", matcher.match(new File("foo")));
        assertTrue("The result should be true", matcher.match(new File("bar")));
        assertFalse("The result should be false", matcher.match(new File("foo.bar")));
        assertTrue("The result should be true", matcher.match(new File("bar.foo")));
    }
}
