/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;
import junit.framework.TestCase;
import java.io.File;


/**
 * <p>
 * Unit test cases for XmlStatsReporting.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class XmlStatsReportingTests extends TestCase {
    /**
     * The the base path of the test files.
     */
    private static final String BASE_PATH = "test_files/reporting";

    /**
     * The namespace for this test.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics.reporting.unittest";

    /**
     * The config file for this test.
     */
    private static final String CONFIG_FILE = "reporting/reporting.xml";

    /**
     * An XmlStatsReporting instance to test.
     */
    private XmlStatsReporting reporting = null;

    /**
     * A Metric instance to test.
     */
    private Metric metric1 = new Metric("metric1", "metric1", "foo");

    /**
     * A Metric instance to test.
     */
    private Metric metric2 = new Metric("metric2", "metric2", "foo");

    /**
     * A Metric instance to test.
     */
    private Metric metric3 = new Metric("metric3", "metric3", "foo");

    /**
     * A StatsResult instance to test.
     */
    private StatsResult result1 = null;

    /**
     * A StatsResult instance to test.
     */
    private StatsResult result2 = null;

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor1 = new TextLineCounter("text", new ExtensionMatcher("txt"));

    /**
     * A FileProcessor instance to test.
     */
    private FileProcessor processor2 = new CStyleLineCounter("java", new ExtensionMatcher("java"));

    /**
     * Creates an XmlStatsReporting instance.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        reporting = new XmlStatsReporting();

        result1 = new StatsResult();
        result2 = new StatsResult();

        result1.setStat(metric1, 10);
        result1.setStat(metric2, 20);
        result2.setStat(metric2, 30);
        result2.setStat(metric3, 0);

        TestHelper.loadConfigFile(CONFIG_FILE, NAMESPACE);
    }

    /**
     * Remove the configuration after testing.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {
        TestHelper.removeNamespace(NAMESPACE);
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(String).
     * </p>
     */
    public void testXmlStatsReporting1() {
        assertTrue("Unable to instantiate XmlStatsReporting.",
            new XmlStatsReporting() instanceof StatsReporting);
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting2() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "reporting1");

        assertTrue("Unable to instantiate XmlStatsReporting.",
            new XmlStatsReporting(property) instanceof StatsReporting);
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(Property).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting3() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "reporting2");

        assertTrue("Unable to instantiate XmlStatsReporting.",
            new XmlStatsReporting(property) instanceof StatsReporting);
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(Property) with null argument.
     * NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting4() throws Exception {
        try {
            new XmlStatsReporting((Property) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting5() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "reporting_invalid1");

        try {
            new XmlStatsReporting(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(Property) with invalid property.
     * ConfigurationException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting6() throws Exception {
        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE,
                "reporting_invalid2");

        try {
            new XmlStatsReporting(property);
            fail("ConfigurationException should be thrown");
        } catch (ConfigurationException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(File).
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting7() throws Exception {
        assertTrue("Unable to instantiate XmlStatsReporting.",
            new XmlStatsReporting(new File("test_files/reporting/xml_to_html.xsl")) instanceof StatsReporting);
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(File) with null argument. NullPointerException
     * should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testXmlStatsReporting8() throws Exception {
        try {
            new XmlStatsReporting((File) null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(File) with invalid file.
     * IllegalArgumentException should be thrown.
     * </p>
     */
    public void testXmlStatsReporting9() {
        try {
            new XmlStatsReporting(new File(BASE_PATH));
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Verifies the constructor - XmlStatsReporting(File) with invalid file.
     * IllegalArgumentException should be thrown.
     * </p>
     */
    public void testXmlStatsReporting10() {
        try {
            new XmlStatsReporting(new File(BASE_PATH, "no_such_file"));
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the getBaseDirectory method.
     * </p>
     */
    public void testGetBaseDirectory1() {
        assertNull("null is expected", reporting.getBaseDirectory());
    }

    /**
     * <p>
     * Tests the getBaseDirectory method.
     * </p>
     */
    public void testGetBaseDirectory2() {
        File base = new File(BASE_PATH);
        reporting.reset(base);
        assertEquals("base is expected", base, reporting.getBaseDirectory());
    }

    /**
     * <p>
     * Tests the aggregate method with base directory.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate1() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "a/b.txt"), result2);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt"), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "b/a.txt"), result2);

        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric1) == 10);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric2) == 50);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric3) == 0);

        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric1) == 20);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric2) == 70);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric3) == 0);

        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric1) == 20);
        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric2) == 40);

        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric1) == 20);
        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric2) == 100);
        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric3) == 0);
    }

    /**
     * <p>
     * Tests the aggregate method without base directory.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate2() throws Exception {
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "a/b.txt"), result2);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt"), result1);

        String file = new File(BASE_PATH, "a/b.txt").getAbsolutePath();
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get(file).getStat(metric1) == 10);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get(file).getStat(metric2) == 50);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get(file).getStat(metric3) == 0);

        String dir = new File(BASE_PATH, "a").getAbsolutePath();
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get(dir).getStat(metric1) == 20);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get(dir).getStat(metric2) == 70);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get(dir).getStat(metric3) == 0);
    }

    /**
     * <p>
     * Tests the aggregate method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate3() throws Exception {
        try {
            reporting.aggregate(null, new File("foo"), result1);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate4() throws Exception {
        try {
            reporting.aggregate(processor1, null, result1);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with null argument. NullPointerException should be thrown.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testAggregate5() throws Exception {
        try {
            reporting.aggregate(processor1, new File("foo"), null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with invalid file. StatsReportingException should be thrown.
     * </p>
     */
    public void testAggregate6() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor1, new File(BASE_PATH), result1);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with invalid file. StatsReportingException should be thrown.
     * </p>
     */
    public void testAggregate7() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor1, new File(BASE_PATH + "foo"), result1);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the aggregate method with invalid file. StatsReportingException should be thrown.
     * </p>
     */
    public void testAggregate8() {
        reporting.reset(new File(BASE_PATH));

        try {
            reporting.aggregate(processor1, new File(BASE_PATH).getParentFile(), result1);
            fail("StatsReportingException should be thrown");
        } catch (StatsReportingException e) {
            // success
        }
    }

    /**
     * <p>
     * Tests the reset method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testReset() throws Exception {
        File base = new File(BASE_PATH);
        reporting.reset(base);
        assertEquals("base is expected", base, reporting.getBaseDirectory());

        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);

        reporting.reset(null);
        assertNull("base is expected", reporting.getBaseDirectory());

        assertNull("file stats is incorrect", reporting.getFileStats().get("a\\b.txt"));
        assertNull("dir stats is incorrect", reporting.getDirectoryStats().get("a"));
        assertNull("alias stats is incorrect", reporting.getAliasStats().get("text"));
    }

    /**
     * <p>
     * Tests the getFileStats method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetFileStats() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);

        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric1) == 10);
        assertTrue("file stats is incorrect",
            reporting.getFileStats().get("a\\b.txt").getStat(metric2) == 20);
    }

    /**
     * <p>
     * Tests the getDirectoryStats method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetDirectoryStats() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);

        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric1) == 10);
        assertTrue("dir stats is incorrect",
            reporting.getDirectoryStats().get("a").getStat(metric2) == 20);
    }

    /**
     * <p>
     * Tests the getAliasStats method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetAliasStats() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);

        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric1) == 10);
        assertTrue("alias stats is incorrect",
            reporting.getAliasStats().get("text").getStat(metric2) == 20);
    }

    /**
     * <p>
     * Tests the getOverallStats method.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGetOverallStats() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);

        assertTrue("overall stats is incorrect", reporting.getOverallStats().getStat(metric1) == 10);
        assertTrue("overall", reporting.getOverallStats().getStat(metric2) == 20);
    }

    /**
     * <p>
     * Tests the generateReport method without stylesheet.  See
     * test_files/reporting/sample_report.xml for more details.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGenerateReport1() throws Exception {
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "a/b.txt"), result2);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt"), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "b/a.txt"), result2);

        String report = reporting.generateReport();

        assertTrue("the report is rendered incorrectly.", report.indexOf("a\\b.txt") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("a\\c.txt") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("b\\a.txt") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("text") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("java") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("metric1") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("metric2") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("metric3") != -1);
    }

    /**
     * <p>
     * Tests the generateReport method without a simple xml-to-html stylesheet. See
     * test_files/reporting/sample_report.xml for more details.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGenerateReport2() throws Exception {
        result1 = new StatsResult();
        result1.setStat(new Metric("line", "line", "line"), 10);

        result2 = new StatsResult();
        result2.setStat(new Metric("line", "line", "line"), 20);

        reporting = new XmlStatsReporting(new File("test_files/reporting/xml_to_html.xsl"));
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt"), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "b/a.txt"), result2);

        String report = reporting.generateReport();

        assertTrue("the report is rendered incorrectly.",
            report.indexOf("File Statistics Report") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("a\\b.txt") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("a\\c.txt") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("b\\a.txt") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("text") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("java") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("line") != -1);
    }

    /**
     * <p>
     * Tests the generateReport method without a simple xml-to-html stylesheet. See
     * test_files/reporting/sample_report.xml for more details.
     * </p>
     *
     * @throws Exception to JUnit
     */
    public void testGenerateReport3() throws Exception {
        result1 = new StatsResult();
        result1.setStat(new Metric("line", "line", "line"), 10);

        result2 = new StatsResult();
        result2.setStat(new Metric("line", "line", "line"), 20);

        Property property = ConfigManager.getInstance().getPropertyObject(NAMESPACE, "reporting2");

        reporting = new XmlStatsReporting(property);
        reporting.reset(new File(BASE_PATH));
        reporting.aggregate(processor1, new File(BASE_PATH, "a/b.txt"), result1);
        reporting.aggregate(processor1, new File(BASE_PATH, "a/c.txt"), result1);
        reporting.aggregate(processor2, new File(BASE_PATH, "b/a.txt"), result2);

        String report = reporting.generateReport();

        assertTrue("the report is rendered incorrectly.",
            report.indexOf("File Statistics Report") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("a\\b.txt") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("a\\c.txt") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("b\\a.txt") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("text") != -1);
        assertTrue("the report is rendered incorrectly.", report.indexOf("java") != -1);

        assertTrue("the report is rendered incorrectly.", report.indexOf("line") != -1);
    }
}
