
package com.topcoder.file.statistics.extractor;

import java.io.File;
import java.io.IOException;

import com.topcoder.file.statistics.ArchiveExtractingException;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.util.archiving.*;
import com.topcoder.util.config.Property;

/**
 * <p>
 * Zip implementation of ArchiveExtractor in this release to delegate the logic to TopCoder Compression
 * Utility.
 * </p>
 * <p> Thread Safety:
 * This class is thread-safe. Each operation works with a new ArchiveUtility instance.
 * </p>
 * 
 * 
 * @author WishingBone
 * @version 2.0c
 * @since 2.0c
 */
public class ZipExtractor extends com.topcoder.file.statistics.extractor.ArchiveExtractorBase {

	/**
	 * <p>
	 * Constructor with all arguments propagated to base constructor.
	 * </p>
	 * 
	 * @param matcher the file matcher
	 * @param pathSeparator the path separator
	 * @exception NullPointerException if either matcher is null
	 */
    public ZipExtractor(FileMatcher matcher, char pathSeparator) {
    	super(matcher, pathSeparator); 
    } 

	/**
	 * <p>
	 * Does nothing more than propagate to base constructor.
	 * </p>
	 * 
	 * @param property the property node to configure from
	 * @exception NullPointerException if property is null
	 * @exception ConfigurationException if config error occurs
	 */
    public ZipExtractor(Property property) throws ConfigurationException {
    	super(property);
    } 

	/**
	 * <p>
	 * Extract file from the archive file. The logic is already well encapsulated by the Compression Utility.
	 * </p>
	 * <algorithm>
	 * 1) Create an ArchiveUtility with the source file and the archiver: new ArchiverUtility(source, new ZipArchiver()).
	 * 2) Extract the files: utility.extractContent(target).
	 * 3) Non IOExceptions are wrapped into ArchiveExtractingException.
	 * </algorithm>
	 * 
	 * 
	 * @param source the sourec archive file
	 * @param target the target directory
	 * @exception NullPointerException if either source or target is null
	 * @exception IOException if io operation fails
	 * @exception ArchiveExtractingException if compression utility fails
	 */
    public void extract(File source, File target) throws IOException, ArchiveExtractingException {
    	if (source == null) {
    		throw new NullPointerException("source should no be null");
    	}
    	if (target == null) {
    		throw new NullPointerException("target should no be null");
    	}
    	ArchiveUtility archiveUtility = new ArchiveUtility(source, new ZipArchiver());
    	
    	try {
    		archiveUtility.extractContents(target);
    	} catch (IllegalArgumentException iae) {
    		throw new ArchiveExtractingException("extracting fails", iae);    	
    	}    	
    } 
 }

