/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileMatchingException;
import com.topcoder.file.type.FileType;
import com.topcoder.file.type.InvalidSignatureException;
import com.topcoder.file.type.InvalidXMLSignatureFileException;
import com.topcoder.file.type.MagicNumbers;
import com.topcoder.util.config.ConfigManagerException;
import java.io.File;
import java.io.IOException;


/**
 * <p>
 * FileTypeMatcher is a template class that handles file type oriented matching logic. File type is
 * resolved by the Magic Numbers component.
 * </p>
 *
 * <p>
 * Thread Safety: This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileTypeMatcher implements FileMatcher {

    /**
     * A MagicNumbers instance. It will be initialized in getMagicNumbers() method and be cached.
     */
    private MagicNumbers magicNumbers = null;

    /**
     * <p>
     * Default protected constructor.
     * </p>
     */
    protected FileTypeMatcher() {
        // empty
    }

    /**
     * <p>
     * It obtains the file type from the Magic Numbers component and uses the template method to
     * decide whether the file matches. The proper configuration of Magic Numbers is not within
     * the responsibility of this class. Any issue around this is translated into
     * FileMatchingException. If the file does not have a correspondant file type, this would
     * result in a false.
     * </p>
     *
     * @param file the file to match
     *
     * @return whether the file matches
     *
     * @throws NullPointerException if file is null
     * @throws FileMatchingException if error occurs
     * @throws IOException if IO operation fails
     */
    public boolean match(File file) throws FileMatchingException, IOException {
        //check for null
        MatcherHelper.checkNull(file, "file");

        // get the type of the given file
        FileType type = getMagicNumbers().determineFileType(file);
        if (type == null) {
            return false;
        }
        // delegate to match(FileType) method
        return match(type);
    }

    /**
     * <p>
     * Gets a MagicNumbers instance.
     * </p>
     *
     * @return a MagicNumbers instance.
     *
     * @throws FileMatchingException if error occurs
     * @throws IOException if IO operation fails
     */
    private MagicNumbers getMagicNumbers() throws FileMatchingException, IOException {
        if (magicNumbers == null) {

            try {
                magicNumbers = new MagicNumbers();
            } catch (InvalidXMLSignatureFileException ixsfe) {
                throw new FileMatchingException("Error occurs while create the MagicNumbers", ixsfe);
            } catch (InvalidSignatureException ise) {
                throw new FileMatchingException("Error occurs while create the MagicNumbers", ise);
            } catch (ConfigManagerException cme) {
                throw new FileMatchingException("Error occurs while create the MagicNumbers", cme);
            }
        }
        return magicNumbers;
    }

    /**
     * <p>
     * Template method that matches the file with file type result.
     * </p>
     *
     * @param type the file type to match
     *
     * @return whether the file matches
     *
     * @throws FileMatchingException if error occurs
     */
    protected abstract boolean match(FileType type) throws FileMatchingException;
}
