/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.matcher;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.type.FileType;
import com.topcoder.util.config.Property;


/**
 * <p>
 * FileTypeMatcher concrete implementation that matches file types with one or more type names (as
 * configured in Magic Numbers).
 * </p>
 *
 * <p>
 * Configuration: This class can be constructed from a Property instance (reflection contract). The
 * supported property is "types" which is potentially multi-valued.
 * </p>
 *
 * <p>
 * Thread Safety: This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class TypeNameMatcher extends FileTypeMatcher {
    /**
     * <p>
     * Configuration key that contains multiple file type names to match. Empty file names are not
     * allowed even if they are valid in the Magic Numbers component.
     * </p>
     */
    public static final String TYPES_PROPERTY_KEY = "types";

    /**
     * <p>
     * A set of file name types (String) to match. This set is initialized in constructor and is
     * immutable.
     * </p>
     */
    private final Set types;

    /**
     * <p>
     * Constructor with single file type name.
     * </p>
     *
     * @param type the file type name
     *
     * @throws NullPointerException if type is null
     * @throws IllegalArgumentException if type is empty
     */
    public TypeNameMatcher(String type) {
        MatcherHelper.checkEmpty(type, "type");
        types = new HashSet();
        types.add(type);
    }

    /**
     * <p>
     * Constructor with multiple file type names.
     * </p>
     *
     * @param types the file type names to match
     *
     * @throws NullPointerException if types is null
     * @throws IllegalArgumentException if types is empty or contains null, non-String instance or
     *         contains empty string (after trimmed)
     */
    public TypeNameMatcher(List types) {
        // check the string list
        MatcherHelper.checkStringList(types, "types");

        // make a shallow copy
        this.types = new HashSet(types);
    }

    /**
     * <p>
     * Constructor that loads type names from a property. It should look for an "types"
     * sub-property and extract multiple values.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if property does not hold a types sub-property or it does not
     *         contain any values or any value is empty after trimmed
     */
    public TypeNameMatcher(Property property) throws ConfigurationException {
        // extract a type string set form given property
        this.types = MatcherHelper.extractStringSet(property, TYPES_PROPERTY_KEY);
    }

    /**
     * <p>
     * Check whether any of the type names match the type. The matching is case-sensitive.
     * </p>
     *
     * @param type the type to match
     *
     * @return whether the type matches
     */
    protected boolean match(FileType type) {
        // check for null
        MatcherHelper.checkNull(type, "type");

        // return whether the type name is in the inner set
        return types.contains(type.getName());
    }
}
