/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.StatsResult;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * StatsCollection is a generic container for mapping from String keys to StatsResult instances. It
 * is used in AbstractStatsReporting to hold file, directory and alias classified mappings. The
 * class provides basic manipulation methods for the mapping.
 * </p>
 *
 * <p>
 * Thread Safety: This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class StatsCollection {
    /**
     * <p>
     * Mapping from String to StatsResult. This is used to classify file, dir and alias basd stats
     * in AbstractStatsReporting. The reference never change.
     * </p>
     */
    private final Map stats = new HashMap();

    /**
     * <p>
     * Default constructor.
     * </p>
     */
    public StatsCollection() {
        // empty
    }

    /**
     * <p>
     * Add a result to specified key. If the mapping already exists, result will be aggregated with
     * this one. Otherwise a clone is saved to the mapping.
     * </p>
     *
     * @param key the key to associate result with
     * @param stats the result to add
     *
     * @throws NullPointerException if key or stats is null
     * @throws IllegalArgumentException if key is empty after trimmed
     */
    public void add(String key, StatsResult stats) {
        if (key == null) {
            throw new NullPointerException("key should not be null.");
        }

        if (key.trim().length() == 0) {
            throw new IllegalArgumentException("key is empty string.");
        }

        if (stats == null) {
            throw new NullPointerException("stats should not be null.");
        }

        // get the result by key
        StatsResult result = (StatsResult) this.stats.get(key);

        if (result == null) {
            // if the result does not exist, add a new one.
            this.stats.put(key, new StatsResult(stats));
        } else {
            // otherwise,  aggregate the result with it
            result.aggregate(stats);
        }
    }

    /**
     * <p>
     * Get the stats result associated with a key. The result is cloned before return. If no result
     * is associated return null.
     * </p>
     *
     * @param key the key to retrieve
     *
     * @return the result associated with key
     *
     * @throws NullPointerException if key is null
     * @throws IllegalArgumentException if key is empty after trimmed
     */
    public StatsResult get(String key) {
        if (key == null) {
            throw new NullPointerException("key should not be null.");
        }

        if (key.trim().length() == 0) {
            throw new IllegalArgumentException("key is empty string.");
        }

        StatsResult result = (StatsResult) stats.get(key);

        if (result == null) {
            // If no result is associated return null.
            return null;
        }
        // otherwise return a new copy
        return new StatsResult(result);
    }

    /**
     * <p>
     * Get the key set of the collection. The return is a shallow copy and sorted in alphabetical
     * order (case insensitive).
     * </p>
     *
     * @return the key set
     */
    public List getAllKeys() {
        // make a shallow copy
        List keys = new ArrayList(stats.keySet());

        // sort the keys in alphabetical order
        Collections.sort(keys, String.CASE_INSENSITIVE_ORDER);

        return keys;
    }

    /**
     * <p>
     * Clear the collection.
     * </p>
     */
    public void clear() {
        stats.clear();
    }
}
